@@ -13,6 +13,7 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 # ----------------------------------------------------------------------------
+import os
 from json import JSONDecodeError
 from typing import List
 from dataclasses import dataclass
@@ -39,9 +40,39 @@ from .authentication import (
     get_basic_auth_header,
 )
 
+# Note: if RAISE_AUTHENTICATION_EXCEPTIONS = False and the client could not
+# authenticate, every function call will return false.
+# Use client.is_active() in order to check authentication status.
+RAISE_AUTHENTICATION_EXCEPTIONS = get_boolean_value_for_env_var(
+    "MONA_SDK_RAISE_AUTHENTICATION_EXCEPTIONS", False
+)
+
+RAISE_EXPORT_EXCEPTIONS = get_boolean_value_for_env_var(
+    "MONA_SDK_RAISE_EXPORT_EXCEPTIONS", False
+)
+
 RAISE_CONFIG_EXCEPTIONS = get_boolean_value_for_env_var(
     "MONA_SDK_RAISE_CONFIG_EXCEPTIONS", False
 )
+
+# Number of retries to authenticate in case the authentication server failed to
+# respond.
+NUM_OF_RETRIES_FOR_AUTHENTICATION = int(
+    os.environ.get("MONA_SDK_NUM_OF_RETRIES_FOR_AUTHENTICATION", 3)
+)
+
+# Time to wait (in seconds) between retries in case the authentication server failed to
+# respond.
+WAIT_TIME_FOR_AUTHENTICATION_RETRIES_SEC = int(
+    os.environ.get("MONA_SDK_WAIT_TIME_FOR_AUTHENTICATION_RETRIES_SEC", 2)
+)
+
+# When this variable is True, failed messages (for any reason) will be logged at "Error"
+# level.
+SHOULD_LOG_FAILED_MESSAGES = get_boolean_value_for_env_var(
+    "MONA_SDK_SHOULD_LOG_FAILED_MESSAGES", False
+)
+
 GET_CONFIG_ERROR_MESSAGE = "Could not get server response with the current config."
 UPLOAD_CONFIG_ERROR_MESSAGE = (
     "Could not upload the new configuration, please check it is valid."
@@ -98,7 +129,17 @@ class Client:
     API.
     """
 
-    def __init__(self, api_key, secret):
+    def __init__(
+        self,
+        api_key,
+        secret,
+        raise_authentication_exceptions=RAISE_AUTHENTICATION_EXCEPTIONS,
+        raise_export_exceptions=RAISE_EXPORT_EXCEPTIONS,
+        raise_config_exceptions=RAISE_CONFIG_EXCEPTIONS,
+        num_of_retries_for_authentication=NUM_OF_RETRIES_FOR_AUTHENTICATION,
+        wait_time_for_authentication_retries=WAIT_TIME_FOR_AUTHENTICATION_RETRIES_SEC,
+        should_log_failed_messages=SHOULD_LOG_FAILED_MESSAGES,
+    ):
         """
         Creates the Client object. this client is lightweight so it can be regenerated
         or reused to user convenience.
@@ -106,8 +147,17 @@ class Client:
         :param secret: The secret corresponding to the given api_key.
         """
         self._logger = get_logger()
-        self._api_key = api_key
-        could_authenticate = first_authentication(api_key, secret)
+        self.api_key = api_key
+        self.secret = secret
+
+        self.raise_authentication_exceptions = raise_authentication_exceptions
+        self.raise_export_exceptions = raise_export_exceptions
+        self.raise_config_exceptions = raise_config_exceptions
+        self.num_of_retries_for_authentication = num_of_retries_for_authentication
+        self.wait_time_for_authentication_retries = wait_time_for_authentication_retries
+        self.should_log_failed_messages = should_log_failed_messages
+
+        could_authenticate = first_authentication(self)
         if not could_authenticate:
             return
 
@@ -124,14 +174,14 @@ class Client:
         Use this method to check client status in case RAISE_AUTHENTICATION_EXCEPTIONS
         is set to False.
         """
-        return is_authenticated(self._api_key)
+        return is_authenticated(self.api_key)
 
     def _get_user_id(self):
         """
         :return: The customer's user id (tenant id).
         """
         decoded_token = jwt.decode(
-            get_current_token_by_api_key(self._api_key), verify=False
+            get_current_token_by_api_key(self.api_key), verify=False
         )
         return decoded_token["tenantId"]
 
@@ -169,14 +219,20 @@ class Client:
         return self._export_batch_inner(events)
 
     def _export_batch_inner(self, events: List[MonaSingleMessage]):
-        events = mona_messages_to_dicts_validation(events)
+        events = mona_messages_to_dicts_validation(
+            events, self.raise_export_exceptions, self.should_log_failed_messages
+        )
         if not events:
             return False
 
         messages_to_send = []
         for message_event in events:
             if not validate_mona_single_message(message_event):
-                return False
+                return handle_export_error(
+                    "Messages to export must be of MonaSingleMessage type.",
+                    self.raise_export_exceptions,
+                    events if self.should_log_failed_messages else None,
+                )
 
             message_copy = dict(message_event)
 
@@ -197,7 +253,11 @@ class Client:
         try:
             rest_api_response = self._send_mona_rest_api_request(messages_to_send)
         except ConnectionError:
-            return handle_export_error("Cannot connect to rest-api")
+            return handle_export_error(
+                "Cannot connect to rest-api",
+                self.raise_export_exceptions,
+                events if self.should_log_failed_messages else None,
+            )
 
         # Create the response and return it.
         client_response = Client._create_client_response(
@@ -206,7 +266,9 @@ class Client:
         )
         if client_response["failed"] > 0:
             handle_export_error(
-                f"Some messages didn't pass validation: {client_response}"
+                f"Some messages didn't pass validation: {client_response}",
+                self.raise_export_exceptions,
+                events if self.should_log_failed_messages else None,
             )
         else:
             self._logger.info(
@@ -223,7 +285,7 @@ class Client:
         return requests.request(
             "POST",
             self._rest_api_url,
-            headers=get_basic_auth_header(self._api_key),
+            headers=get_basic_auth_header(self.api_key),
             json={"userId": self._user_id, "messages": messages},
         )
 
@@ -280,7 +342,7 @@ class Client:
         """
         config_to_upload = {
             "config": {self._user_id: config},
-            "author": self._api_key,
+            "author": self.api_key,
             "commit_message": commit_message,
             "user_id": self._user_id,
         }
@@ -288,7 +350,7 @@ class Client:
         try:
             upload_response = requests.post(
                 f"{self._app_server_url}/upload_config",
-                headers=get_basic_auth_header(self._api_key),
+                headers=get_basic_auth_header(self.api_key),
                 json=config_to_upload,
             )
             response_data = upload_response.json()["response_data"]
@@ -316,7 +378,7 @@ class Client:
         try:
             config_response = requests.post(
                 f"{self._app_server_url}/configs",
-                headers=get_basic_auth_header(self._api_key),
+                headers=get_basic_auth_header(self.api_key),
                 data="{}",
             )
             config_data = config_response.json()
@@ -336,6 +398,6 @@ class Client:
         else returns false.
         """
         self._logger.error(error_message)
-        if RAISE_CONFIG_EXCEPTIONS:
+        if self.raise_config_exceptions:
             raise MonaConfigUploadException(error_message)
         return False
